// --- Buttons --- //
/*
              C1  C2  C3  C4  C5
            +---+---+---+---+---+
    TOP_ROW | 1 | 2 | 3 | 4 | 5 |
            +---+---+---+---+---+
 BOTTOM_ROW | 6 | 7 | 8 | 9 | 10|
            +---+---+---+---+---+
*/



// --- Pin definitions --- //
// Buttons connected in columns starting from the left
#define COL1 10
#define COL2 16
#define COL3 9
#define COL4 5
#define COL5 2

// Rows
#define TOP_ROW 15
#define BOTTOM_ROW 8



// --- Constants --- //
#define DEBOUNCE_TIME 50 //ms



// --- Joystick library --- //
// Include Joystick library
// Refer to: https://github.com/NicoHood/HID
#include "HID-Project.h"



// --- Joystick and pins setup --- //
void setup() { 
  Gamepad.begin();

  // Pin setup:
  // Set column pins as imputs with pullup
  pinMode(COL1, INPUT_PULLUP);
  pinMode(COL2, INPUT_PULLUP);
  pinMode(COL3, INPUT_PULLUP);
  pinMode(COL4, INPUT_PULLUP);
  pinMode(COL5, INPUT_PULLUP);

  // Set row pins as outputs
  pinMode(TOP_ROW, OUTPUT);
  pinMode(BOTTOM_ROW, OUTPUT);

  // Set row pins to high
  digitalWrite(TOP_ROW, HIGH);
  digitalWrite(BOTTOM_ROW, HIGH);
}

// --- Variables --- //
bool button[10] = {0,0,0,0,0,0,0,0,0,0};                // Table containing current button states
bool last_button_state[10] = {0,0,0,0,0,0,0,0,0,0};     // Table containing previous button states
unsigned int change_time[10] = {0,0,0,0,0,0,0,0,0,0};   // Table containing times when last change happened
unsigned int current_time = 0;                          // Scan time 

// --- Main loop --- //
void loop() {
  current_time = millis();
    
  // Scan top row:
  // Set top row pin to low
  digitalWrite(TOP_ROW, LOW);
  // Read pins, if we detect low it means the button is pressed
  if(current_time - change_time[0] > DEBOUNCE_TIME) button[0] = !digitalRead(COL1);
  if(current_time - change_time[1] > DEBOUNCE_TIME) button[1] = !digitalRead(COL2);
  if(current_time - change_time[2] > DEBOUNCE_TIME) button[2] = !digitalRead(COL3);
  if(current_time - change_time[3] > DEBOUNCE_TIME) button[3] = !digitalRead(COL4);
  if(current_time - change_time[4] > DEBOUNCE_TIME) button[4] = !digitalRead(COL5);
  // Revert row pin to high
  digitalWrite(TOP_ROW, HIGH);

  // Scan bottom row:
  digitalWrite(BOTTOM_ROW, LOW);
  if(current_time - change_time[5] > DEBOUNCE_TIME) button[5] = !digitalRead(COL1);
  if(current_time - change_time[6] > DEBOUNCE_TIME) button[6] = !digitalRead(COL2);
  if(current_time - change_time[7] > DEBOUNCE_TIME) button[7] = !digitalRead(COL3);
  if(current_time - change_time[8] > DEBOUNCE_TIME) button[8] = !digitalRead(COL4);
  if(current_time - change_time[9] > DEBOUNCE_TIME) button[9] = !digitalRead(COL5);
  digitalWrite(BOTTOM_ROW, HIGH);

  // Check if any button changed state and update it
  for (int i = 0; i < 10; i++) {              // Iterate over button state table
    if (button[i] != last_button_state[i]) {  // Check if state changed
      //Joystick.setButton(i, button[i]);     // Update Joystick object
      if (button[i]) {                        // Button pressed
        Gamepad.press(i + 1);
      } else {                                // Button released
        Gamepad.release(i + 1);
      }
      last_button_state[i] = button[i];       // Update last state for that button
      change_time[i] = current_time;          // Set change time to current time
    }
  }
  Gamepad.write();                            // Send data to PC at the end of every loop iteration
}
